<?php

declare(strict_types=1);

namespace Drupal\soap_manager\Plugin\QueueWorker;

use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Queue\QueueWorkerBase;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Queue\RequeueException;
use Drupal\vitrina_api\Service\VitrinaApiClient;
use GuzzleHttp\Exception\ConnectException;
use GuzzleHttp\Exception\ServerException;
use GuzzleHttp\Exception\RequestException;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Processes employee dismissal queue items.
 *
 * @QueueWorker(
 *   id = "employee_status_queue",
 *   title = @Translation("Employee Status Queue Worker"),
 *   cron = {"time" = 60}
 * )
 */
class EmployeeStatusQueueWorker extends QueueWorkerBase implements ContainerFactoryPluginInterface {

  /**
   * The logger service.
   */
  protected $logger;

  /**
   * The Vitrina API client.
   */
  protected $apiClient;

  /**
   * Constructs a new EmployeeStatusQueueWorker.
   *
   * @param array $configuration
   *   The plugin configuration.
   * @param string $plugin_id
   *   The plugin ID.
   * @param mixed $plugin_definition
   *   The plugin definition.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_factory
   *   The logger factory service.
   * @param \Drupal\vitrina_api\Service\VitrinaApiClient $api_client
   *   The Vitrina API client service.
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    LoggerChannelFactoryInterface $logger_factory,
    VitrinaApiClient $api_client
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->logger = $logger_factory->get('soap_manager');
    $this->apiClient = $api_client;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(
    ContainerInterface $container,
    array $configuration,
    $plugin_id,
    $plugin_definition
  ) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('logger.factory'),
      $container->get('vitrina_api.client')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function processItem($data) {
    try {
      if (isset($data['user_id'])) {
        $userId = $data['user_id'];
        $this->logger->info('Processing employee dismissal for user ID: @id', [
          '@id' => $userId,
        ]);

        // Update user status to blocked (false)
        $this->updateUserStatus($userId, false);
      }
    }
    catch (ConnectException $e) {
      // Connection error - API is unavailable
      $this->logger->warning('API unavailable when processing dismissal for user ID @id: @message. Will retry later.', [
        '@id' => $data['user_id'] ?? 'unknown',
        '@message' => $e->getMessage(),
      ]);
      // Throw RequeueException to ensure item is processed again later
      throw new RequeueException('API unavailable: ' . $e->getMessage());
    }
    catch (ServerException $e) {
      // Server error (5xx responses) - likely temporary
      $this->logger->warning('API server error when processing dismissal for user ID @id: @message. Will retry later.', [
        '@id' => $data['user_id'] ?? 'unknown',
        '@message' => $e->getMessage(),
      ]);
      // Throw RequeueException to ensure item is processed again later
      throw new RequeueException('API server error: ' . $e->getMessage());
    }
    catch (RequestException $e) {
      // Other request issues that might be temporary
      if ($e->getCode() >= 500) {
        $this->logger->warning('API request failed with code @code for user ID @id: @message. Will retry later.', [
          '@code' => $e->getCode(),
          '@id' => $data['user_id'] ?? 'unknown',
          '@message' => $e->getMessage(),
        ]);
        // Throw RequeueException to ensure item is processed again later
        throw new RequeueException('API request failed: ' . $e->getMessage());
      }
      else {
        // Client errors (4xx) are not likely to be resolved by retrying
        $this->logger->error('API client error for user ID @id: @message', [
          '@id' => $data['user_id'] ?? 'unknown',
          '@message' => $e->getMessage(),
        ]);
        // Don't requeue, this is likely a permanent issue
        throw $e;
      }
    }
    catch (\Exception $e) {
      // Other unexpected errors
      $this->logger->error('Error processing employee dismissal queue item: @message', [
        '@message' => $e->getMessage(),
      ]);
      // Re-throw to allow the queue to handle the failure
      throw $e;
    }
  }

  /**
   * Updates user status in the API.
   *
   * @param int $userId
   *   The user ID to update.
   * @param bool $status
   *   The new status for the user (true for active, false for blocked).
   *
   * @return array
   *   The API response data.
   *
   * @throws \GuzzleHttp\Exception\GuzzleException
   *   If the API request fails.
   */
  protected function updateUserStatus($userId, $status) {
    $response = $this->apiClient->updateUser($userId, ['status' => $status]);

    if (empty($response['status']) || $response['status'] !== $status) {
      $this->logger->error('Failed to update user status for user ID @id: @message', [
        '@id' => $userId,
        '@message' => $response['message'],
      ]);
      throw new \Exception('Failed to update user status');
    }

    $this->logger->info('User ID @id status successfully updated to @status', [
      '@id' => $userId,
      '@status' => $status ? 'active' : 'blocked',
    ]);

    return $response;
  }
}
